tanker = {}

-------------------------------------------------------------------------
--  TankerShoot_v1.00   ---  Slightly Modified Copy of Tanker_v1_04
--                           for use with the Tanker Shoot Mission
--------------------------------------------------------------------------
tanker.Version = "1.00"               -- See bottom of file for changelog
tanker.Asof    = "2023/03"
tanker.DEBUG   = false                -- Control DEBUG messaging during run 
--------------------------------------------------------------------------
--Default tanker settings
--------------------------------------------------------------------------
tanker.GRP_NAME  = nil                -- User can supply these, or we'll make some later
tanker.UNIT_NAME = nil

tanker.AIRCRAFT_TYPE = "S-3B"         -- Can be:  IL-78, KC-130, KC-135, KC-135MPRS, S-3B 
tanker.COUNTRY = "USA"
tanker.ALTITUDE = 8000                -- feet
tanker.TRUE_AIRSPEED = 345            -- knots ( ~265 kts IAS @ 25000 ft)
tanker.CALL_SIGN = "Arco"  
tanker.LIVERY = "default"

tanker.REFERENCE_OBJECT = "CVN-74"    -- anchor object for spawn and racetrack location
tanker.TRACK_START = 3                -- nautical miles from  reference object
tanker.TRACK_LENGTH = 15              -- nautical miles along ref obj heading

tanker.RADIO_FREQUENCY = 251
tanker.USE_TACAN = true
tanker.TACAN_CHANNEL = "51X"
tanker.TACAN_ID = "C74"
tanker.USE_METRIC = false
tanker.RESPAWN = false                -- Re-spawn if RTB or Destroyed
--------------------------------------------------------------------------
--------------------------------------------------------------------------

tanker.totalTankers = 0
tanker.alive =  {}                    -- keep a tabe of live tankers we spawned
tanker.onboard = { ["RED"]    =  1, 
                   ["Arco"]   = 11, 
		   ["Shell"]  = 11, 
		   ["Texaco"] = 11
	         }                    -- VALID Onboard Number Table for each callsign

function tanker.getHeading(unit)
	local unitPosition = unit:getPosition()
	if unitPosition then
           local heading = math.atan2(unitPosition.x.z, unitPosition.x.x)
	   if heading < 0 then 	heading = heading + 2 * math.pi
           end
	   return heading
	end
end


function tanker.getTacanFreq(channel)
	local frequency = 0
	if channel < 64 then  frequency = channel + 1087
	else                  frequency = channel + 961
	end
	return frequency * 1000000
end


function tanker.getOnboardNum(countryID, callsignName)
	 --
	 ---  Retrieve / Assign valid callsign & onboard numbers based on coalition and callsignName
	 --
	 --   NOTE: This function should NOT be called until *after* countryID & callsignName are validated!!!
	 --
	 --   OK... Tanker Numbers are ... weird.  Callsign digits can be 1-9 in the first place and 1-4 in the
	 --   2nd place for BLUE coalition tankers... So, basically we're counting in base 5, and disallowing 
	 --   any cases with a 0 as a digit -- note the example below
	 --   ie.    Shell 1-1, 1-2, 1-3, 1-4, then 2-1 (NOT 1-5, or 2-0).
	 --
         if tanker.DEBUG then
            trigger.action.outText("getOnboardNum: Parms ==> " .. countryID  .. "   " .. callsignName , 40)
         -- trigger.action.outText("getOnboardNum: TABLE ==> " .. "   RED"   .. "   " .. tanker.onboard["RED"]    .. "   ",  40)
         -- trigger.action.outText("getOnboardNum:       ==> " .. "  Arco"   .. "   " .. tanker.onboard["Arco"]   .. "   ",  40)
         -- trigger.action.outText("getOnboardNum:       ==> " .. " Shell"   .. "   " .. tanker.onboard["Shell"]  .. "   ",  40)
         -- trigger.action.outText("getOnboardNum:       ==> " .. "Texaco"   .. "   " .. tanker.onboard["Texaco"] .. "   ",  40)
         end
	 local callsign   = nil
	 local calltype   = nil
	 local onbrdnum   = nil
	 local nextnum    = nil
	 local countryNum = country.id[string.upper(countryID)]

	 if coalition.getCountryCoalition(countryNum) == coalition.side.RED then
            onbrdnum = tanker.onboard["RED"]                --- Assign the current sequence number for RED side (ignore callsignName)
	    callsign = onbrdnum
	    nextnum  = onbrdnum + 1
	    if nextnum > 999 then nextnum = 1   end         --- Update the RED side table
	    tanker.onboard["RED"] = nextnum
            if tanker.DEBUG then
               trigger.action.outText("getOnboardNum: For RED side Tanker ==> callsign = " .. callsign .. "   onboard = " .. onbrdnum , 40)
            end
            return callsign, onbrdnum
	 end

	 if coalition.getCountryCoalition(countryNum) == coalition.side.BLUE then
            onbrdnum = tanker.onboard[callsignName]          --- Retrive / Assign current table number for Tanker Name (Arco, Shell, Texaco)
	    local cs1 = string.sub(tostring(onbrdnum),1,1)   --- take each separate digit of the two digit number as strings 
	    local cs2 = string.sub(tostring(onbrdnum),2,2)   --- Assign that to the callsign table for Blue
	    callsign = 
	    {  [1] = cs1, [2] = cs2,
	       ["name"] = callsignName .. cs1 .. cs2
	    }                                                --- Blue side callsign is a small table  (Red side is just a string -- Go Figure)

	    if     callsignName == "Texaco" then  calltype = 1
            elseif callsignName == "Arco"   then  calltype = 2
            elseif callsignName == "Shell"  then  calltype = 3
            end
            table.insert(callsign, 1, calltype)
	    cs1 = tonumber(cs1)                              --- Let's figure out the next valid number to assign to the next created tanker
            cs2 = tonumber(cs2) + 1                          --- usign base 5 and skipping any zeroes (ie. 11, 12, 13, 14, 21, 22, 23, 24, 31 etc.)
	    if cs2 > 4 then 
	       cs2 = 1
	       cs1 = cs1 + 1
	       if cs1 > 9 then 
		  cs1 = 1
	       end
            end
            nextnum = (cs1 * 10) + cs2 
	    tanker.onboard[callsignName] = nextnum           --- Update the appropriate table entry
            if tanker.DEBUG then
               trigger.action.outText("getOnboardNum: For BLUE side Tanker ==> callsign = " .. callsign["name"] .. "   onboard = " .. onbrdnum .. "  tbl=(" .. callsign[1] .. callsign[2] .. callsign[3] .. ")" , 40)
            end
            return callsign, onbrdnum
	 end
end


function tanker.checkTankers (args, currtime)
        if tanker.DEBUG then
	   trigger.action.outText("checkTankers: @ " .. currtime, 10)
	end

	for tkr, properties in pairs (tanker.alive) do
           local isDead   = false
	   local acType   = properties [1]
	   local group    = properties [16]
	   local unit     = properties [17]
	   local altitude = properties [4]
           local airspeed = properties [5]
           local radio    = properties [8]
           local tacan    = properties [10]
	   local respawn  = properties [15]

	   if tanker.DEBUG then
	      trigger.action.outText("Tanker " .. tkr .. " (" .. acType         .. "): ==> " ..
	                              "Grp / Unit: "  .. group .. " / " .. unit .. "  " ..
	                              "Alt:  ~" .. math.ceil(altitude / 0.305)  .. "  " .. 
	                              "Spd:  ~" .. math.ceil(airspeed / 0.514)  .. "  " ..
	                              "Comm: " .. radio .. " / " .. tacan       .. "  "
	     			     , 10
	     			   )
           end			
	   --			  
	   --- Consider tanker "Dead" if it doesn't exist  OR  it loses <= 3/4 of its assigned altitude --- 
	   --
           if not Unit.getByName(unit) then isDead = true
	   else	   
	      local curralt = Unit.getByName(unit):getPosition().p.y   --- Current altitude in meters
	      if curralt < (altitude / 4)  then     
	         isDead = true
                 Unit.getByName(unit):destroy()
	      end	 
           end
   
	   if isDead  then
	      ------   Make sure  KillCounter Script is loaded before this call!!!  ------
	      
	      killboard.addKill(acType)

	      ----------------------------------------------------------------------------
              if tanker.DEBUG then
		 trigger.action.outText("Tanker " .. unit .. " (" .. acType .. ") " ..  " appears to be DEAD..." , 10)
	      end
              if respawn  then
                 if tanker.DEBUG then
		    trigger.action.outText("Tanker " .. unit .. " (" .. acType .. ") " .. " is re-spawning..." , 10)
		 end
	         local p = properties
	         ---tanker.spawn(aircraftType, referenceObjectName, countryID, altitude, trueAirspeed, trackStart, trackLength, radioFrequency, callsignName, 
		 --              tacanChannel, tacanID, liveryID, tacanUse, metricUse, respawn, groupName, unitName, callsign, onboardNumber)

		 tanker.spawn(p[1],  p[2],  p[3],  p[4],  p[5],  p[6],  p[7],  p[8],  p[9],  p[10], 
		              p[11], p[12], p[13], p[14], p[15], p[16], p[17], p[18], p[19]
			     )
   
     	      else
	         tanker.alive[tkr] = nil        --- No Respawn set, so drop it from our table     
	         if tanker.DEBUG then   
                    trigger.action.outText("Tanker " .. tkr .. " ( " .. unit .. " / " .. acType .. ") " .. " was dropped from tanker table..." , 10)
	         end
              end
           else
	      if tanker.DEBUG then   
                 trigger.action.outText("Tanker " .. unit .. " (" .. acType .. ") " .. " appears to be alive..." , 10)
	      end
           end 
        end
	return currtime + 15
end


function tanker.validateData(aircraftType, referenceObjectName, callsignName, countryID, tacanChannel, tacanUse, groupName, unitName)

	if tanker.DEBUG then   
           trigger.action.outText("validateData: Tanker " .. unitName .. " (" .. aircraftType .. ") ", 10)
	end

	local validData = true

        if Group.getByName(groupName) ~= nil  then
           trigger.action.outText("Error: Tanker Group Name already exists in the mission:  " .. groupName , 10)
	   validData = false
	   return validData
	end

        if Unit.getByName(unitName) ~= nil  then
           trigger.action.outText("Error: Tanker Unit Name already exists in the mission:  " .. unitName , 10)
	   validData = false
	   return validData
	end

	if Unit.getByName(referenceObjectName) == nil then
	   if StaticObject.getByName(referenceObjectName) == nil then
	      trigger.action.outText("Error: Tanker Anchor / Reference object does not exist:  " .. referenceObjectName , 10)
	      validData = false
	      return validData
	   end
	end
	--
        --- NOTE: These names are "tweaked" in .addNew from the User supplied ones to DCS Internal names ---
	--
	if aircraftType ~= "S-3B Tanker" and aircraftType ~= "KC130" and aircraftType ~= "KC-135" and aircraftType ~= "KC135MPRS" and aircraftType ~= "IL-78M" then
	   trigger.action.outText("Error: Invalid tanker aircraft type:  " .. aircraftType , 10)
	   trigger.action.outText("- Must be IL-78, KC-130, KC-135, KC-135MPRS or S-3B", 10)    
	   validData = false
	   return validData
	end

	local countryNumber = country.id[string.upper(countryID)]
	
	if countryNumber ~= nil then

	   if coalition.getCountryCoalition(countryNumber) == coalition.side.BLUE then
	      if callsignName ~= "Texaco" and callsignName ~= "Arco" and callsignName ~= "Shell" then
	         trigger.action.outText("Error: Invalid tanker callsign:  " .. callsignName , 10)
		 trigger.action.outText("- Must be Texaco, Arco, or Shell for Blue coalition", 10)
		 validData = false
	         return validData
	      end
	   elseif coalition.getCountryCoalition(countryNumber) == coalition.side.RED then
	      if tonumber(callsignName) == nil or tonumber(callsignName) < 1 or tonumber(callsignName) > 9 then
	         trigger.action.outText("Error: Invalid tanker callsign:  " .. callsignName , 15)
		 trigger.action.outText("- Must be a number between 0 and 9 for Red coalition", 10)
		 validData = false
	         return validData
	      end
	   else
	      trigger.action.outText("Error: Invalid tanker country / coalition: " .. countryID , 10)
	      validData = false
	      return validData
	   end
	else
	   trigger.action.outText("Error: Invalid tanker country: " .. countryID , 10)
	   validData = false
	   return validData
	end
	
	if tacanUse then
	   local tacanChannelNumber = tonumber(string.sub(tacanChannel, 1, string.len(tacanChannel) - 1))
	   local tacanChannelMode = string.upper(string.sub(tacanChannel, -1))

	   if tacanChannelNumber == nil or tacanChannelNumber < 1 or tacanChannelNumber > 126 or (tacanChannelMode ~= "X" and tacanChannelMode ~= "Y") then
	      trigger.action.outText("Error: Invalid tanker TACAN channel:  " .. tacanChannel , 10)
	      validData = false
	      return validData
	   end
	end
		
	return validData
end


--------------------------------------------------------------------------
--------------------------------------------------------------------------


--function tanker.addNew(aircraftType, referenceObjectName, countryID, altitude, trueAirspeed, trackStart, trackLength, radioFrequency, callsignName, tacanChannel, tacanID, liveryID)


function tanker.addNew(properties)

if properties.DEBUG == true  then  
   tanker.DEBUG = true                                 -- Allow debugging from inside a mission if desired (as a tanker parm)
   trigger.action.outText("addNew: DEBUG Mode is On...", 5)
end

if properties.GRP_NAME == nil      then groupName = tanker.GRP_NAME         else groupName = properties.GRP_NAME           end
if properties.UNIT_NAME == nil     then unitName = tanker.UNIT_NAME         else unitName = properties.UNIT_NAME           end
if properties.AIRCRAFT_TYPE == nil then aircraftType = tanker.AIRCRAFT_TYPE else aircraftType = properties.AIRCRAFT_TYPE   end
if properties.COUNTRY == nil       then countryID = tanker.COUNTRY          else countryID = properties.COUNTRY            end
if properties.ALTITUDE == nil      then altitude = tanker.ALTITUDE          else altitude = properties.ALTITUDE            end
if properties.TRUE_AIRSPEED == nil then trueAirspeed = tanker.TRUE_AIRSPEED else trueAirspeed = properties.TRUE_AIRSPEED   end
if properties.CALL_SIGN == nil     then callsignName = tanker.CALL_SIGN     else callsignName = properties.CALL_SIGN       end
if properties.REFERENCE_OBJECT == nil then referenceObjectName = tanker.REFERENCE_OBJECT else referenceObjectName = properties.REFERENCE_OBJECT end
if properties.TRACK_START == nil   then trackStart = tanker.TRACK_START     else trackStart = properties.TRACK_START       end
if properties.TRACK_LENGTH == nil  then trackLength = tanker.TRACK_LENGTH   else trackLength = properties.TRACK_LENGTH     end
if properties.RADIO_FREQUENCY == nil then radioFrequency = tanker.RADIO_FREQUENCY else radioFrequency = properties.RADIO_FREQUENCY end
if properties.USE_TACAN == nil     then tacanUse = tanker.USE_TACAN         else tacanUse = properties.USE_TACAN           end
if properties.TACAN_CHANNEL == nil then tacanChannel = tanker.TACAN_CHANNEL else tacanChannel = properties.TACAN_CHANNEL   end
if properties.TACAN_ID == nil      then tacanID = tanker.TACAN_ID           else tacanID = properties.TACAN_ID             end
if properties.LIVERY_ID == nil     then liveryID = tanker.LIVERY_ID         else liveryID = properties.LIVERY_ID           end
if properties.USE_METRIC == nil    then metricUse = tanker.USE_METRIC       else metricUse = properties.USE_METRIC         end
if properties.RESPAWN == nil       then respawn = tanker.RESPAWN            else respawn = properties.RESPAWN              end

        tanker.totalTankers = tanker.totalTankers + 1

	if groupName == nil then
           groupName = "Tkr_Grp_" .. tanker.totalTankers
        end
        if unitName == nil then
           unitName = "Tkr_Unit_" .. tanker.totalTankers
        end

	if tanker.DEBUG then   
           trigger.action.outText("addNew: Creating Tanker " .. unitName .. " (" .. aircraftType .. ") ", 10)
	end

	---  Tweak user supplied A/C Types to the internal named a/c types  ---
	if      aircraftType == "KC-130"  then
		aircraftType =  "KC130"
	elseif  aircraftType == "KC-135"  then
		aircraftType =  "KC-135"
	elseif  aircraftType == "KC-135MPRS"  then
		aircraftType =  "KC135MPRS"            -- NOTE: As of 2023/03 this tanker can only use it's baskets and not it's boom...
	elseif  aircraftType == "IL-78"   then
		aircraftType =  "IL-78M"
	elseif  aircraftType == "S-3B"    then
		aircraftType =  "S-3B Tanker"
	end
		
	if tanker.validateData(aircraftType, referenceObjectName, callsignName, countryID, tacanChannel, tacanUse, groupName, unitName) then
  	   if not metricUse then
              altitude = altitude * 0.3048              -- convert feet to meters
	      trueAirspeed = trueAirspeed * 0.514444    -- convert knots to meters per second
	      trackStart = trackStart * 1852            -- convert nm to meters
	      trackLength = trackLength * 1852          -- convert nm to meters
	   end

	   local callsign
	   local onboardNumber
           callsign, onboardNumber = tanker.getOnboardNum(countryID, callsignName)

	   tanker.spawn(aircraftType, referenceObjectName, countryID, altitude, trueAirspeed, trackStart, trackLength, radioFrequency, callsignName, tacanChannel, tacanID, liveryID, tacanUse, metricUse, respawn, groupName, unitName, callsign, onboardNumber)

           local tankerKey = tostring(tanker.totalTankers)     --- Save our spawned tanker in a table for later checking
           tanker.alive[tankerKey] = { aircraftType, referenceObjectName, countryID, altitude, trueAirspeed, trackStart, trackLength, radioFrequency, callsignName, tacanChannel, tacanID, liveryID, tacanUse, metricUse, respawn, groupName, unitName, callsign, onboardNumber  }

	   if tanker.DEBUG then   
              trigger.action.outText("addNew: Tanker " .. unitName .. " (" .. aircraftType .. ") added to table ... Key: " .. tankerKey , 10)
	   end
	else
           if tanker.DEBUG then   
              trigger.action.outText("addNew: Tanker " .. unitName .. " (" .. aircraftType .. ")  FAILED validation -- Not Created...", 10)
	   end
           tanker.totalTankers = tanker.totalTankers - 1
        end
end


function tanker.spawn(aircraftType, referenceObjectName, countryID, altitude, trueAirspeed, trackStart, trackLength, radioFrequency, callsignName, tacanChannel, tacanID, liveryID, tacanUse, metricUse, respawn, groupName, unitName, callsign, onboardNumber)
		
           if tanker.DEBUG then   
              trigger.action.outText("spawn: Spawning Tanker " .. unitName .. " (" .. aircraftType .. ") ", 10)
	   end

	   local referenceObject = Unit.getByName(referenceObjectName)        --- Can't spawn / respawn Tanker if ref obj is destroyed ... Error Out here
	   if referenceObject == nil then   referenceObject = StaticObject.getByName(referenceObjectName)
	      if referenceObject == nil then
                 trigger.action.outText("Reference Object " .. referenceObjectName .. " does NOT exist!!! Unable to spawn Tanker " .. unitName .. " (" .. aircraftType .. ") ", 10)
		 for tkr, props in pairs(tanker.alive) do           -- Since we can't spawn this guy, let's delete his table entry if this was a respawn attempt
		     if groupName == props[16]  and  unitName == props[17] then
			tanker.alive[tkr] = nil
                        trigger.action.outText("Tanker " .. groupName .." / " .. unitName .. " (" .. aircraftType .. ") deleted from tanker table", 10)
		     end
		 end
		 return
	      end
	   end

	   local countryNumber = country.id[string.upper(countryID)]
	   local referencePoint = referenceObject:getPoint()
	   local referenceX = referencePoint.x
	   local referenceY = referencePoint.z		
	   local referenceHeading = tanker.getHeading(referenceObject) % (2 * 3.14159265359)
		
	   local referenceHeadingCos = math.cos(referenceHeading)
	   local referenceHeadingSin = math.sin(referenceHeading)
		
	   local spawnX = (referenceHeadingCos * (trackStart - 926)) + referenceX -- 926 meters = 0.5 nm
	   local spawnY = (referenceHeadingSin * (trackStart - 926)) + referenceY -- 926 meters = 0.5 nm

	   local trackStartX = (referenceHeadingCos * trackStart) + referenceX
	   local trackStartY = (referenceHeadingSin * trackStart) + referenceY

	   local trackEndX = (referenceHeadingCos * (trackStart + trackLength)) + referenceX
	   local trackEndY = (referenceHeadingSin * (trackStart + trackLength)) + referenceY

	   local fuel = 0
	   local flares = 0
	   local chaff = 0
		
	   if aircraftType == "S-3B Tanker" then
	      ---fuel = "8000"
	      fuel   = 17225
	      flares = 30
	      chaff  = 30
	   elseif aircraftType == "KC-135" then
	      ---fuel = 90700
	      fuel   = 199959
	      flares = 0
	      chaff  = 0
	   elseif aircraftType == "KC135MPRS" then
	      ---fuel = 90700
	      fuel   = 199959
	      flares = 60
	      chaff  = 120
	   elseif aircraftType == "KC130" then
	      ---fuel = 20830
	      fuel   = 66139
	      flares = 60
	      chaff  = 120
	   elseif aircraftType == "IL-78M" then
	      ---fuel = 90000
	      fuel   = 198416
	      flares = 96
	      chaff  = 96
	   end
		
           local tacanChannelNumber = 0
	   local tacanChannelMode = ""
	   local tacanCallsign = ""
	   local tacanFrequency = 0
	   local tacanSystem = 0

	   if tacanUse then
	      tacanChannelNumber = tonumber(string.sub(tacanChannel, 1, string.len(tacanChannel) - 1))
	      tacanChannelMode = string.upper(string.sub(tacanChannel, -1))
	      tacanCallsign = string.upper(string.sub(tacanID, 1, 3))
	      tacanFrequency = tanker.getTacanFreq(tacanChannelNumber)
              if tacanChannelMode == "X"     then   tacanSystem = 4
	      elseif tacanChannelMode == "Y" then   tacanSystem = 5
	      end
	   end

           local groupData = 
	   {
		["modulation"] = 0,
		["tasks"] = 
		{
	        }, -- end of ["tasks"]
	        ["radioSet"] = false,
	        ["task"] = "Refueling",
	        ["uncontrolled"] = false,
	        ["route"] = 
	        {
	          ["points"] = 
	          {
	            [1] = 
	            {
		      ["alt"] = altitude,
		      ["action"] = "Turning Point",
		      ["alt_type"] = "BARO",
		      ["speed"] = trueAirspeed,
		      ["task"] = 
		      {
		        ["id"] = "ComboTask",
		        ["params"] = 
		        {
			  ["tasks"] = 
		    	  {
		            [1] = 
			    {
			      ["number"] = 1,
			      ["auto"] = true,
			      ["id"] = "Tanker",
			      ["enabled"] = true,
			      ["params"] = 
			      {
		              }, -- end of ["params"]
		            }, -- end of [1]
		            [2] = 
		            {
		              ["enabled"] = true,
		              ["auto"] = true,
		              ["id"] = "WrappedAction",
		              ["number"] = 2,
		              ["params"] = 
		              {
		                ["action"] = 
		                {
		                  ["id"] = "ActivateBeacon",
			          ["params"] = 
			          {
			            ["type"] = 4,
			            ["AA"] = false,
			            ["callsign"] = tacanCallsign,
			            ["system"] = tacanSystem,
			            ["channel"] = tacanChannelNumber,
			            ["modeChannel"] = tacanChannelMode,
			            ["bearing"] = true,
			            ["frequency"] = tacanFrequency
			          }, -- end of ["params"]
		                 }, -- end of ["action"]
		                }, -- end of ["params"]
		              }, -- end of [2]
		            }, -- end of ["tasks"]
	                  }, -- end of ["params"]
			}, -- end of ["task"]
			["type"] = "Turning Point",
			["y"] = spawnY,
			["x"] = spawnX,
			["name"] = "Wpt_1",
			["formation_template"] = "",
			["speed_locked"] = true,
		       }, -- end of [1]
		       [2] = 
		       {
		         ["alt"] = altitude,
			 ["action"] = "Turning Point",
			 ["alt_type"] = "BARO",
			 ["speed"] = trueAirspeed,
			 ["task"] = 
			 {
			   ["id"] = "ComboTask",
			   ["params"] = 
			   {
			     ["tasks"] = 
			     {
			       [1] = 
			       {
			         ["number"] = 1,
				 ["auto"] = false,
				 ["id"] = "Orbit",
				 ["enabled"] = true,
				 ["params"] = 
				 {
				   ["altitude"] = altitude,
				   ["pattern"] = "Race-Track",
				   ["speed"] = trueAirspeed,
				   ["speedEdited"] = true,
				 }, -- end of ["params"]
			        }, -- end of [1]
			      }, -- end of ["tasks"]
			    }, -- end of ["params"]
			   }, -- end of ["task"]
			   ["type"] = "Turning Point",
			   ["y"] = trackStartY,
			   ["x"] = trackStartX,
			   ["name"] = "Wpt_2",
			   ["formation_template"] = "",
			   ["speed_locked"] = true,
			 }, -- end of [2]
			 [3] = 
			 {
			   ["alt"] = altitude,
			   ["action"] = "Turning Point",
			   ["alt_type"] = "BARO",
			   ["speed"] = trueAirspeed,
			   ["task"] = 
			   {
			     ["id"] = "ComboTask",
			     ["params"] = 
			     {
			       ["tasks"] = 
			       {
			         [1] = 
				 {
				   ["number"] = 1,
				   ["auto"] = false,
				   ["id"] = "WrappedAction",
				   ["enabled"] = true,
				   ["params"] = 
				   {
				     ["action"] = 
				     {
				       ["id"] = "Option",
				       ["params"] = 
				       {
				         ["value"] = true,
					 ["name"] = 6,
				       }, -- end of ["params"]
				      }, -- end of ["action"]
				    }, -- end of ["params"]
				   }, -- end of [1]
				  }, -- end of ["tasks"]
				}, -- end of ["params"]
			      }, -- end of ["task"]
			      ["type"] = "Turning Point",
			      ["y"] = trackEndY,
			      ["x"] = trackEndX,
			      ["name"] = "Wpt_3",
			      ["formation_template"] = "",
			      ["speed_locked"] = true,
			    }, -- end of [3]
			    [4] = 
			    {
			      ["alt"] = altitude / 2,
			      ["action"] = "Turning Point",
			      ["alt_type"] = "BARO",
			      ["speed"] = trueAirspeed,
			      ["task"] = 
			      {
			        ["id"] = "ComboTask",
				["params"] = 
				{
				  ["tasks"] = 
				  {
				  }, -- end of ["tasks"]
				}, -- end of ["params"]
			       }, -- end of ["task"]
			       ["type"] = "Turning Point",
			       ["y"] = referenceY,
			       ["x"] = referenceX,
			       ["name"] = "Wpt_4",
			       ["formation_template"] = "",
			       ["speed_locked"] = true,
			     }, -- end of [4]
			}, -- end of ["points"]
		     }, -- end of ["route"]
		     --["groupId"] = 9999,
		     ["hidden"] = false,
		     ["units"] = 
		     {
		       [1] = 
		       {
		         ["alt"] = altitude,
		         ["alt_type"] = "BARO",
		         ["livery_id"] = liveryID,
		         ["skill"] = "Excellent",
		         ["speed"] = trueAirspeed,
		         ["type"] = aircraftType,
		         ["psi"] = -referenceHeading,
		         ["y"] = spawnY,
		         ["x"] = spawnX,
		         ["name"] = unitName,
		         ["payload"] = 
		         {
		           ["pylons"] = 
		           {
		           }, -- end of ["pylons"]
		           ["fuel"] = fuel,
		           ["flare"] = flares,
		           ["chaff"] = chaff,
		           ["gun"] = 100,
		         }, -- end of ["payload"]
		         ["heading"] = referenceHeading,
		         ["callsign"] = callsign,
		         ["onboard_num"] = onboardNumber,
	               }, -- end of [1]
		     }, -- end of ["units"]
		     ["y"] = spawnY,
		     ["x"] = spawnX,
		     ["name"] = groupName,
		     ["communication"] = true,
		     ["start_time"] = 0,
		     ["frequency"] = radioFrequency,
	     } -- end of groupData

	    coalition.addGroup(countryNumber, Group.Category.AIRPLANE, groupData)

	    local callout = nil
	    if type(callsign) == "table"  then
	       callout = string.sub(callsign.name, 1, -3) .. ":" .. callsign[2] .. "-" .. callsign[3]
	    else 
	       callout = callsign
            end
	    local onStationAltitude = altitude
	    if not metricUse then
	       onStationAltitude = math.floor(onStationAltitude * 3.28084) -- convert meters to feet
	    end
	    trigger.action.outTextForCoalition(coalition.getCountryCoalition(countryNumber), "Tanker " .. groupName .. " / " .. unitName .. " (" .. aircraftType .. ") is on station as " .. callout .. " at " .. onStationAltitude .. " on " .. radioFrequency .. " MHz / " .. tacanChannel, 20)

end
	

do
	trigger.action.outText("Tanker Script  v" .. tanker.Version .. " (" .. tanker.Asof .. ") -- Loaded... ", 5)

	if tanker.DEBUG then
	   trigger.action.outText("DEBUG Mode is On...", 5)
	end

	--tanker.addNew("IL-78M", "Tire1", "Russia", 20000, 300, 20, 30, 270, 5, "10X", true)
	--tanker.addNew("KC-130", "CVN-70", "USA", 15000, 300, 1, 20, 251, "Texaco", "12X", true)

	timer.scheduleFunction(tanker.checkTankers, nil, timer.getTime() + 15)    -- check the status of our tankers every 15 seconds

end


---------------------------------------------------------------------------------------------------
----------                         Script Changelog & Notes                              ----------
---------------------------------------------------------------------------------------------------
--  VER      WHEN       WHAT
---------------------------------------------------------------------------------------------------
--  1.00   2023/03/20   Copied Tanker_v1_04  to customize for Marianas Tanker Shoot fun mission.
--                      added call to KillCounter. Changed alt parameter b4 considered dead
--
--
---------------------------------------------------------------------------------------------------
